#include "mbed.h"
#include "system.h"
#include "acc.h"



#define ACC_SDA   0,0
#define ACC_SCL   0,1
#define ACC_INT_B 0,11

#define ACC_I2C_ADDR (0x4c)



long acc_x,acc_y,acc_z;
int acc_i2c_status,acc_i2c_phase;
uint8_t acc_command[3]={0x07,0x01,0x00};
uint8_t acc_status[16];



int acc_i2c_write(int length, uint8_t *data);
int acc_i2c_read(int length, uint8_t *data);
int acc_i2c_write_byte(unsigned char data);
unsigned char acc_i2c_read_byte(int last);

void acc_init(void)
   {
   system_setbitpair(&LPC_SC->PCLKSEL1,6,1);        // 6 = PCLK_I2C1 (1 = CCLK)
   LPC_I2C1->I2CONSET=0x00000040;                   // enable
   LPC_I2C1->I2CONCLR=0x00000038;                   // clear STA, STO and SI
   LPC_I2C1->I2SCLL=480;                            // low clock phase 5 us - the MMA7660 is not specified to be compatible with a symmetric 400 kHz clock!!!
   LPC_I2C1->I2SCLH=480;                            // high clock phase 5 us

   system_configurepin(ACC_SDA, 3, 6, 0, 0);     // p46, P0.0, 3 (SDA1), no pull-up or pull-down, open drain
   system_configurepin(ACC_SCL, 3, 6, 0, 0);     // p47, P0.1, 3 (SCL1), no pull-up or pull-down, open drain
   
   system_configurepin(ACC_INT_B, 0, 0, 0, 0);   // p49, P0.11, 0 (GPIO), pull-up, input
   
   wait_us(10);
   acc_i2c_write(3,acc_command);
   }



// the accelerometer signal is very noisy
void acc_measure(void)
   {
   acc_i2c_read(11,acc_status);
   if ((acc_status[0]&0x40)==0) acc_x = (acc_status[0]&0x20) ? 0xffffffe0|(acc_status[0]&0x1f) : acc_status[0]&0x1f;
   if ((acc_status[1]&0x40)==0) acc_y = (acc_status[1]&0x20) ? 0xffffffe0|(acc_status[1]&0x1f) : acc_status[1]&0x1f;
   if ((acc_status[2]&0x40)==0) acc_z = (acc_status[2]&0x20) ? 0xffffffe0|(acc_status[2]&0x1f) : acc_status[2]&0x1f;
   }



// The documentation at the LPC1768 User Manual made litte sense
// It seems that clearing the SI bit is the magic kick for the internal
// state machine. If no special bits are set, the data at IDAT is transmitted.
// Once again: if the SI bit is set, nothing happens until it is cleared,
// therefore the data and STA/STO bits have to be set first.
// The first STA happens immediately, as the SI bit is not set.
// Multiple STO do not do anything, so they can safely be used to reset the state.
// The waits are ugly, but double as timeouts. If something goes wrong here, let it.
int acc_i2c_write(int length, uint8_t *data)
   {
   int i;
   if (((LPC_I2C1->I2STAT)&0xf8)!=0xf8)
      {
      LPC_I2C1->I2CONSET=0x00000010;    // send STO to handle previous (unlikely, but possible) errors
      LPC_I2C1->I2CONCLR=0x00000028;    // clear STA, and SI
      wait_us(10);                      // if there is no pending error, nothing has happened this far
      }
   LPC_I2C1->I2CONSET=0x00000020;       // start
   wait_us(10);                         // no other masters, this cannot take longer
   //if (((LPC_I2C1->I2STAT)&0xf8)!=0x08) return(1);   // start ok?
   LPC_I2C1->I2DAT=(ACC_I2C_ADDR<<1);    // write address + 'write'-bit
   LPC_I2C1->I2CONCLR=0x00000028;   // clear STA and SI
   wait_us(100);
   //if (((LPC_I2C1->I2STAT)&0xf8)!=0x18) return(2);   // address acknowledged ok?
   for(i=0;i<length;i++)
      {
      acc_i2c_write_byte(*data);
      data++;
      }
   LPC_I2C1->I2CONSET=0x00000010;   // set STO
   LPC_I2C1->I2CONCLR=0x00000008;   // clear SI
   return(0);
   }
   
   
   
int acc_i2c_read(int length, uint8_t *data)
   {
   unsigned char hb;
   int i;
   acc_i2c_status=0;
   if (((LPC_I2C1->I2STAT)&0xf8)!=0xf8)
      {
      LPC_I2C1->I2CONSET=0x00000010;    // send STO to handle previous (unlikely, but possible) errors
      LPC_I2C1->I2CONCLR=0x00000028;    // clear STA, and SI
      wait_us(10);                      // if there is no pending error, nothing has happened this far
      }
   LPC_I2C1->I2CONSET=0x00000020;       // start
   wait_us(10);                     // no other masters, this cannot take longer
   if (((LPC_I2C1->I2STAT)&0xf8)!=0x08) acc_i2c_status=1;   // start ok?
   LPC_I2C1->I2DAT=(ACC_I2C_ADDR<<1)|1;    // write address + 'read'-bit
   LPC_I2C1->I2CONCLR=0x00000028;   // clear STA and SI
   wait_us(100);
   if (((LPC_I2C1->I2STAT)&0xf8)!=0x40) acc_i2c_status=2;   // address acknowledged ok?
   for(i=0;i<length;i++)
      {
      *data=acc_i2c_read_byte(i==length-1);
      data++;
      }
   LPC_I2C1->I2CONSET=0x00000010;   // set STO
   LPC_I2C1->I2CONCLR=0x00000008;   // clear SI
   return(0);
   }
   
   
   
int acc_i2c_write_byte(unsigned char data)
   {
   LPC_I2C1->I2DAT=data;
   LPC_I2C1->I2CONCLR=0x00000008;   // clear SI
   wait_us(100);
   //if (((LPC_I2C1->I2STAT)&0xf8)!=0x28) return(1);   // data acknowledged ok?
   return(0);
   }
   
   
   
unsigned char acc_i2c_read_byte(int last)
   {
   unsigned char data;
   if (last)
      {
      LPC_I2C1->I2CONCLR=0x0000000c;    // clear SI and AA (to produce NACK)
      }
   else
      {
      LPC_I2C1->I2CONSET=0x00000004;    // set AA (to produce ACK)
      LPC_I2C1->I2CONCLR=0x00000008;    // clear SI
      }
   wait_us(100);                        // assume no clock stretching
   if (((LPC_I2C1->I2STAT)&0xf8)!=0x50) acc_i2c_status=3; 
   data=LPC_I2C1->I2DAT;
   return(data);
   }
   
            


