#include "mbed.h"
#include "system.h"
#include "display.h"
#include "touch.h"
#include "camera.h"
#include "image.h"
#include "phone.h"
#include "vista.h"
#include "canvas.h"

enum CANVAS_MODE {CANVAS_MODE_START,CANVAS_MODE_THERMAL,CANVAS_MODE_INPUT,CANVAS_MODE_CALL,CANVAS_MODE_INCOMING};
enum CANVAS_INPUT_USE {CANVAS_INPUT_PIN,CANVAS_INPUT_CALL,CANVAS_INPUT_MMS};

long canvas_mode=CANVAS_MODE_START,canvas_input_use,canvas_input_maxlength,canvas_input_allowplus,canvas_returnmode,canvas_returnmode2;
long canvas_redraw;
long canvas_x,canvas_y,canvas_down,canvas_click,canvas_time;
long canvas_buttonorder[12]={1,2,3,4,5,6,7,8,9,10,0,13};

unsigned char canvas_number[16];
char canvas_asciinumber[16];
long canvas_numberlength=0;
char *canvas_input_title="",*canvas_input_green="",*canvas_input_red="";

char *canvas_number2ascii();



void canvas_init(void)
   {
   canvas_redraw=1;
   canvas_down=0;
   canvas_mode=CANVAS_MODE_START;
   }



// the magic values for the touch point calculation were measured for the display type used
// and seem to work quite well from unit to unit   
void canvas_update(void)
   {
   unsigned long i,j,k,a,b,c;
   a=b=c=0;
   canvas_time=LPC_TIM3->TC;
   // check stylus
   for(i=0;i<16;i++)
      {
      touch_measure();
      a+=touch_x;
      b+=touch_y;
      c+=touch_z;
      }
   touch_x=a/16;
   touch_y=b/16;
   touch_z=c/16;
   // 3890... 220
   canvas_x=((3890-touch_x)*240)/3670;
   canvas_x=LIMIT(canvas_x,0,239);
   // 400... 3920
   canvas_y=((touch_y-400)*320)/3520;
   canvas_y=LIMIT(canvas_y,0,319);
   canvas_click=0; // click happens for one cycle when the touch threshold is exceeded
   if (canvas_down)
      {
      if (touch_z>2500)
         {
         canvas_down=0;
         }    
      }
   else
      {
      if (touch_z<2000) // this seems to be critical
         {
         canvas_down=1;
         canvas_click=1;
         }    
      }
   // asking PIN has the highest priority - just wait for the possible previous input to happen
   if ((phone_pinready()==2)&&(canvas_mode!=CANVAS_MODE_INPUT)) // PIN request most likely happens at the very beginning, but it can happen later, too, if the gsm module reboots by itself
      {
      canvas_returnmode2=canvas_returnmode; // fortunately, the stack is max 2 deep
      canvas_returnmode=canvas_mode;
      canvas_input_use=CANVAS_INPUT_PIN;
      canvas_input_maxlength=4;
      canvas_input_allowplus=0;
      canvas_input_title="PIN";
      canvas_input_green="SET";
      canvas_input_red="";
      canvas_mode=CANVAS_MODE_INPUT;
      canvas_redraw=1;
      }  
   // incoming phone call has the second highest priority - obviously this cannot happen during the PIN input
   if ((phone_ringing())&&(canvas_mode!=CANVAS_MODE_INCOMING)&&(canvas_mode!=CANVAS_MODE_CALL))
      {
      canvas_returnmode2=canvas_returnmode; // fortunately, the stack is max 2 deep
      canvas_returnmode=canvas_mode;
      canvas_mode=CANVAS_MODE_INCOMING;
      canvas_redraw=1;
      }
   // remote end hanging up during call has similar priority
   if (((canvas_mode==CANVAS_MODE_CALL)||(canvas_mode==CANVAS_MODE_INCOMING))&&(!phone_live()))
      {
      canvas_returnmode=canvas_returnmode2;
      canvas_mode=canvas_returnmode;
      canvas_redraw=1;
      }
   // user input comes last :-( 
   if (canvas_click)
      {
      switch (canvas_mode) 
         {
         case CANVAS_MODE_START :              // starting screen
            if ((canvas_x>57)&&(canvas_x<57+74)&&(canvas_y>23)&&(canvas_y<23+53)) // phone booth
               {
               // make phone call
               if (phone_ready())
                  { 
                  canvas_numberlength=0;
                  canvas_input_use=CANVAS_INPUT_CALL;
                  canvas_input_maxlength=14;
                  canvas_input_allowplus=1;
                  canvas_returnmode=CANVAS_MODE_START;
                  canvas_input_title="CALL TO";
                  canvas_input_green="CALL";
                  canvas_input_red="CANCEL";
                  canvas_mode=CANVAS_MODE_INPUT;
                  canvas_redraw=1;
                  }
               }
            else if ((canvas_x>33)&&(canvas_x<33+51)&&(canvas_y>214)&&(canvas_y<214+92)) // gallery ad
               {
               // send multimedia message
               if ((camera_snapshot)&&(phone_ready()))
                  {
                  canvas_numberlength=0;
                  canvas_input_use=CANVAS_INPUT_MMS;
                  canvas_input_maxlength=14;
                  canvas_input_allowplus=1;
                  canvas_returnmode=CANVAS_MODE_START;
                  canvas_input_title="SEND TO";
                  canvas_input_green="SEND";
                  canvas_input_red="CANCEL";
                  canvas_mode=CANVAS_MODE_INPUT;
                  canvas_redraw=1;
                  }
               }
            else if ((canvas_x>128)&&(canvas_x<128+26)&&(canvas_y>192)&&(canvas_y<192+50)) // movie billboard
               {
               // view thermal image
               canvas_mode=CANVAS_MODE_THERMAL;
               canvas_redraw=1;
               }               
            break;
         case CANVAS_MODE_THERMAL :              // thermal image
            camera_snap();
            canvas_mode=CANVAS_MODE_START;
            canvas_redraw=1;
            break;
         case CANVAS_MODE_INPUT: // input a number
            if (canvas_y>128)
               {
               i=canvas_x/48;
               j=(canvas_y-128)/48;
               if ((i>0)&&(i<4)) // numberpad area
                  {
                  k=j*3+(i-1);
                  k=canvas_buttonorder[k];
                  if ((canvas_numberlength>0)&&(k==13)) // backspace
                     {
                     canvas_numberlength--;
                     canvas_redraw=1;
                     } 
                  if ((canvas_numberlength<canvas_input_maxlength)&&(k<11))
                     {
                     if ((k!=10)||((canvas_numberlength==0)&&(canvas_input_allowplus))) // allow 'plus' only in the first place
                        {
                        canvas_number[canvas_numberlength]=k; 
                        canvas_numberlength++;
                        canvas_redraw=1;
                        }
                     }
                  }
               else  // call or cancel boxes
                  {
                  if ((i==0)&&(canvas_input_use!=CANVAS_INPUT_PIN)) // cancel (no cancel for PIN request - should there be...?)
                     {
                     canvas_mode=CANVAS_MODE_START;
                     canvas_redraw=1;
                     }
                  else // call or send
                     {
                     if (canvas_input_use==CANVAS_INPUT_MMS)
                        {
                        phone_sendMMS(canvas_number2ascii());
                        canvas_mode=CANVAS_MODE_START;
                        canvas_redraw=1;
                        }
                     else if (canvas_input_use==CANVAS_INPUT_CALL)
                        {
                        phone_call(canvas_number2ascii());
                        canvas_returnmode=CANVAS_MODE_START;
                        canvas_mode=CANVAS_MODE_CALL;
                        canvas_redraw=1;
                        }
                     else if (canvas_input_use==CANVAS_INPUT_PIN)
                        {
                        phone_setpin(canvas_number2ascii());
                        canvas_returnmode=canvas_returnmode2;
                        canvas_mode=canvas_returnmode;
                        canvas_redraw=1;
                        }
                     else // error, should not be possible
                        {
                        canvas_mode=CANVAS_MODE_START;
                        canvas_redraw=1;
                        }
                     }
                  }
               }
            else
               {
               canvas_mode=CANVAS_MODE_START;
               canvas_redraw=1;
               }
            break;
         case CANVAS_MODE_CALL :              // phone call in progress
            if (canvas_y>160)
               {
               phone_hangup();
               canvas_mode=canvas_returnmode;
               canvas_redraw=1;
               }
            break;
         case CANVAS_MODE_INCOMING :              // incoming call
            if (canvas_y>160)
               {
               if (canvas_x<120) // cancel
                  {
                  phone_hangup();
                  canvas_mode=canvas_returnmode;
                  canvas_returnmode=canvas_returnmode2;
                  canvas_redraw=1;
                  }
               else // accept
                  {
                  phone_answer();
                  canvas_mode=CANVAS_MODE_CALL;
                  canvas_redraw=1;
                  }
               }
            break;
          default :             // should never happen
            break;
         }
      }
   // update static portions of the display
   if (canvas_redraw)
      {
      switch (canvas_mode) 
         {
         case CANVAS_MODE_START :              // starting screen
            display_rleimage(0,0,240,320,(uint8_t *)vista_rle,(uint16_t *)vista_palette);
            if (camera_snapshot) display_camerahalfimage(44,237,30,40,camera_index_snap,camera_palette);             
            break;
         case CANVAS_MODE_THERMAL :              // thermal image
            break;
         case CANVAS_MODE_INPUT :              // input a number
            display_clear(0x0000);
            display_box(4,32,232,64,display_RGB2color(64,64,64)); // number display
            display_zoomedRGBtext(120,48,1,20,0,canvas_input_title,64,64,64,255,255,255);
            canvas_number2ascii();
            display_zoomedRGBtext(120,80,1,20,0,canvas_asciinumber,64,64,64,0,255,0);
            display_box(4,128+4,40,184,display_RGB2color(255,128,128)); // cancel
            display_zoomedRGBtext(24,224,1,0,20,canvas_input_red,255,128,128,255,255,255);
            display_box(192+4,128+4,40,184,display_RGB2color(128,255,128)); // call or send
            display_zoomedRGBtext(216,224,1,0,20,canvas_input_green,128,255,128,255,255,255);
            for(j=0;j<4;j++)
               for(i=0;i<3;i++)
                  {
                  display_box(48+4+i*48,128+4+j*48,40,40,display_RGB2color(128,128,128));
                  display_zoomedblendedRGBimage(48+8+i*48,128+8+j*48,16,16,2,(uint8_t *)image_button[canvas_buttonorder[3*j+i]],64,64,64,255,255,255);
                  }
            break;
         case CANVAS_MODE_CALL : // call in progress
            display_clear(0x0000);
            display_box(4,32,232,64,display_RGB2color(64,64,64)); // text box
            display_zoomedRGBtext(120,64,1,16,0,"CALL ACTIVE",64,64,64,255,255,255);
            display_box(4,160+4,232,152,display_RGB2color(255,128,128)); // hang up
            display_zoomedRGBtext(120,240,1,16,0,"HANG UP",255,128,128,255,255,255);
            break;
         case CANVAS_MODE_INCOMING : // incoming phone call
            display_clear(0x0000);
            display_box(4,32,232,64,display_RGB2color(64,64,64)); // text box
            display_zoomedRGBtext(120,64,1,16,0,"INCOMING CALL",64,64,64,255,255,255);
            display_box(4,160+4,112,152,display_RGB2color(255,128,128)); // reject
            display_zoomedRGBtext(60,240,1,16,0,"REJECT",255,128,128,255,255,255);
            display_box(124,160+4,112,152,display_RGB2color(128,255,128)); // accept
            display_zoomedRGBtext(180,240,1,16,0,"ACCEPT",128,255,128,255,255,255);
            break;
         default :             // should never happen
            break;
         }
      canvas_redraw=0;
      }
   // update dynamic content
   switch (canvas_mode) 
      {
      case CANVAS_MODE_START :              // starting screen
         if (camera_newimage)
            {
            camera_newimage=0;
            camera_index();
            display_cameraimage(128,192,26,50,camera_index_image,camera_palette);          
            system_setled(rand()&0x40);
            }          
         break;
      case CANVAS_MODE_THERMAL :              // thermal image
         if (camera_newimage)
            {
            camera_newimage=0;
            camera_index();
            display_fullscreencameraimage(camera_index_image,camera_palette);
            }
         break;
      default :             // should never happen
         break;
      }        
   canvas_time=LPC_TIM3->TC-canvas_time;
   }


char *canvas_number2ascii()
   {
   int i;
   for(i=0;i<canvas_numberlength;i++)
      {
      if (canvas_number[i]==10) // plus
         {
         canvas_asciinumber[i]='+';
         }
      else
         {
         canvas_asciinumber[i]='0'+canvas_number[i];
         }
      }
   canvas_asciinumber[canvas_numberlength]=0;
   return(canvas_asciinumber);
   }
