#include "mbed.h"
#include <string.h>
#include "system.h"
#include "display.h"
#include "image.h"
#include "ili9341.h"

uint16_t display_palette[256];

#define LCD_WR_B   1,14
#define LCD_RS    1,9
#define LCD_RST_B 1,15
#define LCD_CS_B  1,10
#define LCD_BL    2,11


void display_init(void)
   {
   uint16_t i;
   // LCD_WR_B
   system_configurepin(LCD_WR_B, 0, 0, 1, 1);   // p89, P1.14, 0 (GPIO), pull-up, output, 1
   
   // LCD_RS
   system_configurepin(LCD_RS, 0, 0, 1, 1);     // p91, P1.9, 0 (GPIO), pull-up, output, 1
   
   // LCD_RST_B
   system_configurepin(LCD_RST_B, 0, 0, 1, 1);  // p88, P1.15, 0 (GPIO), pull-up, output, 1

   // LCD_CS_B
   system_configurepin(LCD_CS_B, 0, 0, 1, 1);   // p90, P1.10, 0 (GPIO), pull-up, output, 1

   // LCD_BL
   system_configurepin(LCD_BL, 0, 0, 1, 1);     // p52, P2.11, 0 (GPIO), pull-up, output, 1
   
   // LCD_D[0..15]
   for(i=0;i<16;i++) system_configurepin(1, 16+i, 0, 0, 1, 1); //
   // reset
   wait_ms(10);
   CLRIO(LCD_RST_B);
   wait_ms(10);
   SETIO(LCD_RST_B);
   wait_ms(150);
   
   display_send_command(ILI9341_COMMAND_SLEEP_OUT,1);
   wait_ms(150);
   
   display_send_command(ILI9341_COMMAND_DISPLAY_ON,1);
   
   // 16 bits per pixel
   display_send_command(ILI9341_COMMAND_PIXEL_FORMAT_SET,0);
   display_send_data(0x05,1);

   // set orientation - reverse y (to start from top) and refresh direction (to minimize tearing) - RGB
   display_send_command(ILI9341_COMMAND_MEMORY_ACCESS_CONTROL,0);
   display_send_data(0x98,1);

   // write LUT
   display_send_command(ILI9341_COMMAND_COLOR_SET,0);
   for (i=0;i<32;i++) display_send_data(i<<1,0); // r
   for (i=0;i<64;i++) display_send_data(i,0); // g
   for (i=0;i<32;i++) display_send_data(i<<1,(i==31) ? 1 : 0); // b
   
   display_clear(0x0000);
   }
   


uint16_t display_RGB2color(uint8_t r,uint8_t g,uint8_t b)
   {
   return(((r>>3)<<11)|((g>>2)<<5)|((b>>3)));
   }



void display_setwindow(uint16_t x, uint16_t y, uint16_t w, uint16_t h)
   {
   uint16_t dim;
   display_send_command(ILI9341_COMMAND_COLUMN_ADDRESS_SET,0);
   dim=x;
   display_send_data(dim>>8,0);
   display_send_data(dim&0xff,0);
   dim=x+w-1;
   display_send_data(dim>>8,0);
   display_send_data(dim&0xff,1);
   display_send_command(ILI9341_COMMAND_PAGE_ADDRESS_SET,0);
   dim=y;
   display_send_data(dim>>8,0);
   display_send_data(dim&0xff,0);
   dim=y+h-1;
   display_send_data(dim>>8,0);
   display_send_data(dim&0xff,1);
   }
   
   

void display_image(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t *data)
   {
   uint32_t i;
   display_setwindow(x,y,w,h);
   display_send_command(ILI9341_COMMAND_MEMORY_WRITE,0);
   for (i=0;i<w*h;i++)
      {
      LPC_GPIO1->FIOCLR=0xffff0000;
      CLRIO(LCD_WR_B);
      LPC_GPIO1->FIOSET=((*data++)<<16);
      SETIO(LCD_WR_B);
      }
   SETIO(LCD_CS_B);
   }
   
   

void display_indexedimage(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint8_t *data, uint16_t *palette)
   {
   uint32_t i;
   display_setwindow(x,y,w,h);
   display_send_command(ILI9341_COMMAND_MEMORY_WRITE,0);
   for (i=0;i<w*h;i++)
      {
      LPC_GPIO1->FIOCLR=0xffff0000;
      CLRIO(LCD_WR_B);
      LPC_GPIO1->FIOSET=(palette[*data++]<<16);
      SETIO(LCD_WR_B);
      }
   SETIO(LCD_CS_B);
   }



// using the 16 x 16 font - zoom applies to the increments as well
void display_zoomedRGBtext(uint16_t midx, uint16_t midy, uint16_t z, uint16_t incx, uint16_t incy, char * text, uint8_t r1, uint8_t g1, uint8_t b1, uint8_t r2, uint8_t g2, uint8_t b2)
   {
   int length,i,x,y;
   char c;
   length=strlen(text);
   x=midx-z*(16+(length-1)*incx)/2;
   y=midy-z*(16+(length-1)*incy)/2;
   for(i=0;i<length;i++)
      {
      c=*text++;
      if (c!=' ') display_zoomedblendedRGBimage(x,y,16,16,z,(uint8_t *)image_button[display_locatecharacter(c)],r1,g1,b1,r2,g2,b2);
      x+=z*incx;
      y+=z*incy;
      }
   }
   
   
   
uint8_t display_locatecharacter(char c)
   {
   if ((c>='0')&&(c<='9')) return(c-'0');
   if ((c>='A')&&(c<='Z')) return(c-'A'+16);
   if ((c>='a')&&(c<='z')) return(c-'a'+16); // convert to upper case
   if (c=='+') return(10);
   if (c=='-') return(11);
   if (c==':') return(12);
   // the backspace character is number 13
   if (c=='(') return(14);
   if (c==')') return(15);
   return(11); // - for all unknown characters  
   }



// the width and height (w and h) refer to the source size, the zoom does not affect them
void display_zoomedRGBimage(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t z, uint8_t *data)
   {
   long i,j,ii,jj;
   uint8_t *dataline;
   unsigned char d;
   uint16_t color;
   display_setwindow(x,y,w*z,h*z);
   display_send_command(ILI9341_COMMAND_MEMORY_WRITE,0);
   for(j=0;j<h;j++)
      {
      dataline=data;
      for(jj=0;jj<z;jj++)
         {
         data=dataline;
         for(i=0;i<w;i++)
            {
            d=*data++;
            color=display_RGB2color(d,d,d);
            LPC_GPIO1->FIOCLR=0xffff0000;
            LPC_GPIO1->FIOSET=color<<16;
            for(ii=0;ii<z;ii++)
               { 
               CLRIO(LCD_WR_B);
               SETIO(LCD_WR_B);
               }
            }
         }
      }
   SETIO(LCD_CS_B);
   }
   
   

// the width and height (w and h) refer to the source size, the zoom does not affect them
void display_zoomedblendedRGBimage(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t z, uint8_t *data, uint8_t r1, uint8_t g1, uint8_t b1, uint8_t r2, uint8_t g2, uint8_t b2)
   {
   long i,j,ii,jj;
   uint8_t *dataline;
   unsigned char d;
   uint16_t color;
   display_setwindow(x,y,w*z,h*z);
   display_send_command(ILI9341_COMMAND_MEMORY_WRITE,0);
   for(j=0;j<h;j++)
      {
      dataline=data;
      for(jj=0;jj<z;jj++)
         {
         data=dataline;
         for(i=0;i<w;i++)
            {
            d=*data++;
            color=display_RGB2color((d*r1+(255-d)*r2)>>8,(d*g1+(255-d)*g2)>>8,(d*b1+(255-d)*b2)>>8);
            LPC_GPIO1->FIOCLR=0xffff0000;
            LPC_GPIO1->FIOSET=color<<16;
            for(ii=0;ii<z;ii++)
               { 
               CLRIO(LCD_WR_B);
               SETIO(LCD_WR_B);
               }
            }
         }
      }
   SETIO(LCD_CS_B);
   }
   
   
      
// this function is tailored for the camera image
void display_fullscreencameraimage(uint8_t *data, uint16_t *palette)
   {
   uint16_t x=0,y=0,w=240,h=320;
   uint32_t line[240];
   uint32_t i,j;
   display_setwindow(x,y,w,h);

   display_send_command(ILI9341_COMMAND_MEMORY_WRITE,0);
   for(i=0;i<80;i++)
      {
      for(j=0;j<60;j++) line[j*4+3]=line[j*4+2]=line[j*4+1]=line[j*4+0]=(palette[data[j*80+79-i]]<<16);
      for (j=0;j<240;j++)
         {
         LPC_GPIO1->FIOCLR=0xffff0000;
         CLRIO(LCD_WR_B);
         LPC_GPIO1->FIOSET=line[j];
         SETIO(LCD_WR_B);
         }
      for (j=0;j<240;j++)
         {
         LPC_GPIO1->FIOCLR=0xffff0000;
         CLRIO(LCD_WR_B);
         LPC_GPIO1->FIOSET=line[j];
         SETIO(LCD_WR_B);
         }
      for (j=0;j<240;j++)
         {
         LPC_GPIO1->FIOCLR=0xffff0000;
         CLRIO(LCD_WR_B);
         LPC_GPIO1->FIOSET=line[j];
         SETIO(LCD_WR_B);
         }
      for (j=0;j<240;j++)
         {
         LPC_GPIO1->FIOCLR=0xffff0000;
         CLRIO(LCD_WR_B);
         LPC_GPIO1->FIOSET=line[j];
         SETIO(LCD_WR_B);
         }
      }
   SETIO(LCD_CS_B);
   }
   
   
   
// this function is tailored for the camera image
void display_halfscreencameraimage(uint16_t x, uint16_t y, uint8_t *data, uint16_t *palette)
   {
   uint16_t w=120,h=160;
   uint32_t line[120];
   uint32_t i,j;
   display_setwindow(x,y,w,h);

   display_send_command(ILI9341_COMMAND_MEMORY_WRITE,0);
   for(i=0;i<80;i++)
      {
      for(j=0;j<60;j++) line[j*2+1]=line[j*2+0]=(palette[data[j*80+79-i]]<<16);
      for (j=0;j<120;j++)
         {
         LPC_GPIO1->FIOCLR=0xffff0000;
         CLRIO(LCD_WR_B);
         LPC_GPIO1->FIOSET=line[j];
         SETIO(LCD_WR_B);
         }
      for (j=0;j<120;j++)
         {
         LPC_GPIO1->FIOCLR=0xffff0000;
         CLRIO(LCD_WR_B);
         LPC_GPIO1->FIOSET=line[j];
         SETIO(LCD_WR_B);
         }
      }
   SETIO(LCD_CS_B);
   }
   
   

void display_cameraimage(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint8_t *data, uint16_t *palette)
   {
   uint32_t i,j;
   display_setwindow(x,y,w,h);

   display_send_command(ILI9341_COMMAND_MEMORY_WRITE,0);
   for(i=0;i<h;i++)
      {
      for (j=0;j<w;j++)
         {
         LPC_GPIO1->FIOCLR=0xffff0000;
         CLRIO(LCD_WR_B);
         LPC_GPIO1->FIOSET=palette[data[j*80+79-i]]<<16;
         SETIO(LCD_WR_B);
         }
      }
   SETIO(LCD_CS_B);
   }
       
       

void display_camerahalfimage(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint8_t *data, uint16_t *palette)
   {
   uint32_t i,j;
   uint32_t a,b,c,d,f;
   display_setwindow(x,y,w,h);

   display_send_command(ILI9341_COMMAND_MEMORY_WRITE,0);
   for(i=0;i<h;i++)
      {
      for (j=0;j<w;j++)
         {
         LPC_GPIO1->FIOCLR=0xffff0000;
         CLRIO(LCD_WR_B);
         a=palette[data[(2*j)*80+79-(2*i)]];
         b=palette[data[(2*j)*80+79-(2*i+1)]];
         c=palette[data[(2*j+1)*80+79-(2*i)]];
         d=palette[data[(2*j+1)*80+79-(2*i+1)]];
         f=((((a&0xf800)+(b&0xf800)+(c&0xf800)+(d&0xf800))>>2)&0xf800)|((((a&0x07e0)+(b&0x07e0)+(c&0x07e0)+(d&0x07e0))>>2)&0x07e0)|((((a&0x001f)+(b&0x001f)+(c&0x001f)+(d&0x001f))>>2)&0x001f);
         LPC_GPIO1->FIOSET=f<<16;
         SETIO(LCD_WR_B);
         }
      }
   SETIO(LCD_CS_B);
   }
   
   
   
void display_rleimage(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint8_t *data, uint16_t *palette)
   {
   uint32_t i;
   uint32_t rlecount=0;
   
   // temporarily change data writing direction to match the rle direction
   display_send_command(ILI9341_COMMAND_MEMORY_ACCESS_CONTROL,0);
   display_send_data(0xb8,1);

   display_setwindow(y,x,h,w); // notice the reverse order

   display_send_command(ILI9341_COMMAND_MEMORY_WRITE,0);
   for(i=0;i<w*h;i++)
      {
      if (rlecount) // interestingly, rle is not only smaller, but also faster
         {
         CLRIO(LCD_WR_B);
         rlecount--;
         SETIO(LCD_WR_B);
         }
      else
         {
         rlecount=*(data++);      
         LPC_GPIO1->FIOCLR=0xffff0000;
         CLRIO(LCD_WR_B);
         LPC_GPIO1->FIOSET=palette[*(data++)]<<16;
         SETIO(LCD_WR_B);
         rlecount--;
         }
      }
   SETIO(LCD_CS_B);
   
   // restore data writing direction
   display_send_command(ILI9341_COMMAND_MEMORY_ACCESS_CONTROL,0);
   display_send_data(0x98,1);
   }
   
   
             
void display_box(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t color)
   {
   if (((long)x+w>240)||((long)y+h>320)) return; // the values cannot be negative
   uint32_t i;
   display_setwindow(x,y,w,h);

   display_send_command(ILI9341_COMMAND_MEMORY_WRITE,0);
   LPC_GPIO1->FIOCLR=0xffff0000;
   LPC_GPIO1->FIOSET=(color<<16);
   i=w*h;
   while (i)
      {
      CLRIO(LCD_WR_B);
      i--;
      SETIO(LCD_WR_B);
      }
   SETIO(LCD_CS_B);
   }
    
    
      
void display_clear(uint16_t color)
   {
   display_box(0,0,240,320,color);
   }
    
    
    
// use FIOSET and FIOCLR, as reading FIOPIN is very unreliable  
void display_send_command(uint16_t command, uint8_t finish)
   {
   CLRIO(LCD_CS_B);
   CLRIO(LCD_RS);
   LPC_GPIO1->FIOCLR=0xffff0000;
   LPC_GPIO1->FIOSET=(command<<16);
   CLRIO(LCD_WR_B);
   CLRIO(LCD_WR_B);
   SETIO(LCD_WR_B);
   SETIO(LCD_RS);
   if (finish) SETIO(LCD_CS_B);
   }


   
void display_send_data(uint16_t data, uint8_t finish)
   {
   LPC_GPIO1->FIOCLR=0xffff0000;
   LPC_GPIO1->FIOSET=(data<<16);
   CLRIO(LCD_WR_B);
   CLRIO(LCD_WR_B);
   SETIO(LCD_WR_B);
   if (finish) SETIO(LCD_CS_B);
   }
   
   
      