#include "mbed.h"

#include "system.h"

#define SYSTEM_DEBUG_B  2,10
#define SYSTEM_LED0     1,4

void setbit(volatile std::uint32_t *address, long index, long value);
void setbitpair(volatile std::uint32_t *address, long index, long value);
uint8_t system_debugpin(void);
void system_setled(uint8_t value);


void system_init(void)
   {
   // after the MBED setup the main clock is 96 MHz, which is fine for us
   // power, clock and pin allocation to the peripherals
   LPC_SC->PCONP=0x00e8949c; // TIMER3, TIMER2, SSP0, I2C1, GPIO, ADC, SSP1, I2C0, UART1, UART0, TIMER1 - The value set by MBED is 0x04e887de 
   
   system_configurepin(SYSTEM_DEBUG_B, 0, 0, 0, 0);     // p53, P2.10, 0 (GPIO), pull-up, input, 0
   system_configurepin(SYSTEM_LED0, 0, 0, 1, 0);     // p93, P1.4, 0 (GPIO), pull-up, output,0   
   // Errata states that PLL0 must not be enabled at peripheral clock selection?!?!?!?!
   }
   
   
   
// this function assumes that the control register or structure addresses are consecutive
// not using (unreliable) FIOPIN any more
void system_configurepin(long port, long bit, long function, long mode, long direction, long value)
   {
   long pairport,pairbit;
   pairport=2*port+((bit&0x10)>>4);
   pairbit=(bit&0x0f)<<1;
   system_setbitpair(((&LPC_PINCON->PINSEL0)+pairport),pairbit,function);
   system_setbitpair(((&LPC_PINCON->PINMODE0)+pairport),pairbit,mode);
   system_setbit(((&LPC_PINCON->PINMODE_OD0)+port),bit,(mode>>2)&0x01);
   if (function==0) // GPIO
      {
      system_setbit((&LPC_GPIO0->FIODIR)+port*((&LPC_GPIO1->FIODIR)-(&LPC_GPIO0->FIODIR)),bit,direction);
      if (direction==1) // output
         {
         if (value==1) *((&LPC_GPIO0->FIOSET)+port*((&LPC_GPIO1->FIOSET)-(&LPC_GPIO0->FIOSET)))=(1<<bit);
         else *((&LPC_GPIO0->FIOCLR)+port*((&LPC_GPIO1->FIOCLR)-(&LPC_GPIO0->FIOCLR)))=(1<<bit);
         }
      }
   }



uint8_t system_debugpin(void)
   {
   return((LPC_GPIO2->FIOPIN&0x00000400) ? 0 : 1);
   }
   
   
   
void system_setled(uint8_t value)
   {
   value ? SETIO(SYSTEM_LED0) : CLRIO(SYSTEM_LED0);
   }
   
   
      
void system_setbit(volatile std::uint32_t *address, long index, long value)
   {
   *address|=(*address&(0xffffffff-(1<<index)))|(value<<index);
   }



void system_setbitpair(volatile std::uint32_t *address, long index, long value)
   {
   *address|=(*address&(0xffffffff-(3<<index)))|(value<<index);
   }



