#include "mbed.h"
#include "system.h"
#include "touch.h"

#define LCD_TOUCH_CS_B  0,16
#define LCD_SCK         0,15
#define LCD_MISO        0,17
#define LCD_MOSI        0,18

unsigned int touch_x,touch_y,touch_z,touch_z1,touch_z2;

void touch_init(void)
   {
   // the touch interface uses the SSP0 SPI peripheral
   system_configurepin(LCD_SCK, 2, 0, 0, 0);    // p62, P0.15, 2 (SCK0), pull-up, input, 0
   system_configurepin(LCD_MISO, 2, 0, 0, 0);    // p61, P0.17, 2 (MISO0), pull-up, input, 0
   system_configurepin(LCD_MOSI, 2, 0, 1, 1);    // p60, P0.18, 2 (MOSI0), pull-up, output, 1

   // LCD_TOUCH_CS_B
   system_configurepin(LCD_TOUCH_CS_B, 0, 0, 1, 1);    // p63, P0.16, 0 (GPIO), pull-up, output, 1
    
   // clock
   system_setbitpair(&LPC_SC->PCLKSEL1,10,1); // 10 = PCLK_SSP0 (1 = CCLK)
   // peripheral registers
   LPC_SSP0->CPSR=96; // try 1 MHz clock (max for XPT2046 appears to be 2.5 MHz). The documentation claims this to be the actual divisor, not divisor-1
   LPC_SSP0->CR0=0x00000007; // 8-bit transfers, SPOL = 0, SPHA = 0
   LPC_SSP0->CR1=0x00000002; // enable
   }


   
void touch_measure(void)
   {
   uint8_t a,b,c;
   uint16_t dz;
   CLRIO(LCD_TOUCH_CS_B);
   LPC_SSP0->DR=0x93; // Y
   LPC_SSP0->DR=0x00;
   LPC_SSP0->DR=0x00;
   while (LPC_SSP0->SR&0x00000010);
   a=LPC_SSP0->DR;
   b=LPC_SSP0->DR;
   c=LPC_SSP0->DR;
   touch_y=(b<<5)|(c>>3);

   LPC_SSP0->DR=0xb3; // Z1
   LPC_SSP0->DR=0x00;
   LPC_SSP0->DR=0x00;
   while (LPC_SSP0->SR&0x00000010);
   a=LPC_SSP0->DR;
   b=LPC_SSP0->DR;
   c=LPC_SSP0->DR;
   touch_z1=(b<<5)|(c>>3);

   LPC_SSP0->DR=0xc3; // Z2
   LPC_SSP0->DR=0x00;
   LPC_SSP0->DR=0x00;
   while (LPC_SSP0->SR&0x00000010);
   a=LPC_SSP0->DR;
   b=LPC_SSP0->DR;
   c=LPC_SSP0->DR;
   touch_z2=(b<<5)|(c>>3);

   LPC_SSP0->DR=0xd0; // X
   LPC_SSP0->DR=0x00;
   LPC_SSP0->DR=0x00;
   while (LPC_SSP0->SR&0x00000010);
   a=LPC_SSP0->DR;
   b=LPC_SSP0->DR;
   c=LPC_SSP0->DR;
   touch_x=(b<<5)|(c>>3);
   SETIO(LCD_TOUCH_CS_B);
   dz=touch_z2-touch_z1;
   touch_z=(dz*(1.77*(4095-touch_y)+touch_x)/(4096-dz));
   touch_z>>=2;
   if (touch_z>0x0fff) touch_z=0x0fff; // limit value
   }
   
   
   